@@ -34,7 +34,7 @@ class TestHighlightInline(util.MdCase):
 class TestNoClass(util.MdCase):
     """Test no class."""
 
-    extension = ['pymdownx.highlight', 'pymdownx.superfences']
+    extension = ['pymdownx.highlight', 'pymdownx.superfences', 'markdown.extensions.attr_list']
     extension_configs = {
         'pymdownx.highlight': {
             'css_class': ''
@@ -127,6 +127,43 @@ class TestNoClassNoPygments(util.MdCase):
         )
 
 
+class TestCustomLangPrefixNoPygments(util.MdCase):
+    """Test custom language prefix."""
+
+    extension = ['pymdownx.highlight', 'pymdownx.superfences', 'pymdownx.inlinehilite']
+    extension_configs = {
+        'pymdownx.highlight': {
+            'language_prefix': 'lang-',
+            'use_pygments': False
+        }
+    }
+
+    def test_custom_prefix_no_pygments(self):
+        """Test with custom prefix and no Pygments."""
+
+        self.check_markdown(
+            r'''
+            ```python
+            import test
+            test.test()
+            ```
+            ''',
+            r'''
+            <pre class="highlight"><code class="lang-python">import test
+            test.test()</code></pre>
+            ''',  # noqa: E501
+            True
+        )
+
+    def test_custom_prefix_no_pygments_inline(self):
+        """Test with custom prefix and no Pygments with inline code."""
+
+        self.check_markdown(
+            '`#!python import test`',
+            '<p><code class="lang-python highlight">import test</code></p>'
+        )
+
+
 class TestNoPygments(util.MdCase):
     """Test no Pygments."""
 
@@ -165,8 +202,9 @@ class TestNoPygments(util.MdCase):
             ```
             ''',
             r'''
-            <pre class="highlight"><code class="language-python linenums">import test
-            test.test()</code></pre>
+            <p><code>python linenums="1"
+            import test
+            test.test()</code></p>
             ''',
             True
         )
@@ -198,7 +236,7 @@ class TestNoPygmentsCustomLineClass(util.MdCase):
             ''',
             r'''
             <p>Text</p>
-            <pre class="highlight"><code class="line-numbers">import test
+            <pre class="highlight"><code>import test
             test.test()
             </code></pre>
 
@@ -218,7 +256,7 @@ class TestNoPygmentsCustomLineClass(util.MdCase):
             ```
             ''',
             r'''
-            <pre class="highlight"><code class="language-python line-numbers">import test
+            <pre class="highlight"><code class="language-python">import test
             test.test()</code></pre>
             ''',
             True
@@ -271,3 +309,62 @@ class TestHighlightSpecial(util.MdCase):
             ''',  # noqa: E501
             True
         )
+
+
+class TestDisabledLinenums(util.MdCase):
+    """Test with line numbers globally disabled."""
+
+    extension = ['pymdownx.highlight', 'pymdownx.superfences']
+    extension_configs = {
+        'pymdownx.highlight': {
+            'linenums': False
+        }
+    }
+
+    def test_global_disable(self):
+        """Test with line numbers globally disabled."""
+
+        self.check_markdown(
+            r'''
+            ```python linenums="1"
+            import test
+            test.test()
+            ```
+            ''',
+            r'''
+            <div class="highlight"><pre><span></span><code><span class="kn">import</span> <span class="nn">test</span>
+            <span class="n">test</span><span class="o">.</span><span class="n">test</span><span class="p">()</span>
+            </code></pre></div>
+            ''',  # noqa: E501
+            True
+        )
+
+
+class TestDisabledLinenumsNoPygments(util.MdCase):
+    """Test with line numbers globally disabled with no Pygments."""
+
+    extension = ['pymdownx.highlight', 'pymdownx.superfences']
+    extension_configs = {
+        'pymdownx.highlight': {
+            'linenums': False,
+            'use_pygments': False
+        }
+    }
+
+    def test_global_disable(self):
+        """Test with line numbers globally disabled and no Pygments."""
+
+        self.check_markdown(
+            r'''
+            ```python linenums="1"
+            import test
+            test.test()
+            ```
+            ''',
+            r'''
+            <p><code>python linenums="1"
+            import test
+            test.test()</code></p>
+            ''',  # noqa: E501
+            True
+        )
